library ieee;
use ieee.std_logic_1164.all; 

library systemPll;
library adcPll;
library adcDdrRx_2Ch;
library adcDdrRx_1Ch;

entity MWE is
	port
	(
		-- Clock & Reset --
		RESET_N				: in std_logic;
		ENCODE_CLOCK 		: in std_logic;
		ADC_2CH_CLKOUT 	: in std_logic;
		ADC_1CH_CLKOUT 	: in std_logic;
		
		-- ADC Data
		ADC_2CH_DATA_CHA 	: in std_logic_vector(5 downto 0);
		ADC_2CH_DATA_CHB 	: in std_logic_vector(5 downto 0);
		ADC_2CH_OF 			: in std_logic;
		
		ADC_1CH_DATA 		: in std_logic_vector(5 downto 0);
		ADC_1CH_OF 			: in std_logic;
		
		
		-- Output value to avoid that uses all input values in order to avoid logic being removed
		OUTPUT				: out std_logic
	);
end entity MWE;

architecture rtl of MWE is
	
	--- Unregistered interconnect signals ---	
	
	-- Clock & reset
	signal systemClock			: std_logic;
	signal adc2ChClkout_90deg 	: std_logic;
	signal adc1ChClkout_90deg 	: std_logic;
	
	
	-- ADC Data
	signal adc2ChDataA		: std_logic_vector(11 downto 0);
	signal adc2ChOfA			: std_logic;
	signal adc2ChDataB		: std_logic_vector(11 downto 0);
	signal adc2ChOfB			: std_logic;
	
	signal adc1ChData			: std_logic_vector(11 downto 0);
	signal adc1ChOf			: std_logic;
	signal adc1ChOf_notUsed	: std_logic;

begin
	
	systemPll_inst : entity systemPll.systemPll(rtl)
	port map (
		refclk => ENCODE_CLOCK,
		rst => not RESET_N,
		outclk_0 => systemClock
	);	
--	systemClock <= ENCODE_CLOCK;
		
	-- Clock domain crossing occurs here, not relevant for MWE
	process(RESET_N, systemClock)
		variable tempOutput	: std_logic;
	begin
		if RESET_N = '0' then
			OUTPUT <= '0';
		elsif rising_edge(systemClock) then
			-- All input bits are XORed together to make sure all inputs are used and nothing gets optimized away
			tempOutput := '0';
		
			for i in adc2ChDataA'range loop
				tempOutput := tempOutput xor adc2ChDataA(i);
			end loop;
			
			for i in adc2ChDataB'range loop
				tempOutput := tempOutput xor adc2ChDataB(i);
			end loop;
			
			tempOutput := tempOutput xor adc2ChOfA;
			tempOutput := tempOutput xor adc2ChOfB;
			
			for i in adc1ChData'range loop
				tempOutput := tempOutput xor adc1ChData(i);
			end loop;
			
			tempOutput := tempOutput xor adc1ChOf;
			
			OUTPUT <= tempOutput;
			
			end if;
	end process;
	
	
	--- Instantiate 2CH ADC + PLL ---
	
	adcPll_2Ch_inst : entity adcPll.adcPll(rtl)
	port map (
		refclk => ADC_2CH_CLKOUT,
		rst => not RESET_N,
		outclk_0 => adc2ChClkout_90deg
	);

	adcDdrRx_2Ch_inst : entity adcDdrRx_2Ch.adcDdrRx_2Ch(syn)
	port map (		
		datain => ADC_2CH_DATA_CHA & ADC_2CH_DATA_CHB & ADC_2CH_OF,
		inclock => adc2ChClkout_90deg,
		--inclock => ADC_2CH_CLKOUT,
		dataout_l(12) => adc2ChDataA(0),
		dataout_l(11) => adc2ChDataA(2),
		dataout_l(10) => adc2ChDataA(4),
		dataout_l(9) => adc2ChDataA(6),
		dataout_l(8) => adc2ChDataA(8),
		dataout_l(7) => adc2ChDataA(10),
		dataout_l(6) => adc2ChDataB(0),
		dataout_l(5) => adc2ChDataB(2),
		dataout_l(4) => adc2ChDataB(4),
		dataout_l(3) => adc2ChDataB(6),
		dataout_l(2) => adc2ChDataB(8),
		dataout_l(1) => adc2ChDataB(10),
		dataout_l(0) => adc2ChOfA,
		dataout_h(12) => adc2ChDataA(1),
		dataout_h(11) => adc2ChDataA(3),
		dataout_h(10) => adc2ChDataA(5),
		dataout_h(9) => adc2ChDataA(7),
		dataout_h(8) => adc2ChDataA(9),
		dataout_h(7) => adc2ChDataA(11),
		dataout_h(6) => adc2ChDataB(1),
		dataout_h(5) => adc2ChDataB(3),
		dataout_h(4) => adc2ChDataB(5),
		dataout_h(3) => adc2ChDataB(7),
		dataout_h(2) => adc2ChDataB(9),
		dataout_h(1) => adc2ChDataB(11),
		dataout_h(0) => adc2ChOfB
	);
	
	
	--- Instantiate 1CH ADC + PLL ---
	
	adcPll_1Ch_inst : entity adcPll.adcPll(rtl)
	port map (
		refclk => ADC_1CH_CLKOUT,
		rst => not RESET_N,
		outclk_0 => adc1ChClkout_90deg
	);
	
	adcDdrRx_1Ch_inst : entity adcDdrRx_1Ch.adcDdrRx_1Ch(syn)
	port map (
		datain => ADC_1CH_DATA & ADC_1CH_OF,
		inclock => adc1ChClkout_90deg,
		--inclock => ADC_1CH_CLKOUT,
		dataout_l(6) => adc1ChData(0),
		dataout_l(5) => adc1ChData(2),
		dataout_l(4) => adc1ChData(4),
		dataout_l(3) => adc1ChData(6),
		dataout_l(2) => adc1ChData(8),
		dataout_l(1) => adc1ChData(10),
		dataout_l(0) => adc1ChOf,
		dataout_h(6) => adc1ChData(1),
		dataout_h(5) => adc1ChData(3),
		dataout_h(4) => adc1ChData(5),
		dataout_h(3) => adc1ChData(7),
		dataout_h(2) => adc1ChData(9),
		dataout_h(1) => adc1ChData(11),
		dataout_h(0) => adc1ChOf_notUsed
	);
	
end rtl;